/*
   pikt.c -- PIKT script interpreter

   $Header: /home/cvs/ucgsb/pikt/src/piktd/pikt.c,v 1.9 1999/04/13 01:20:07 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA

   --------------------------------------------------------------------------

   Usage:  pikt [-cGhT] [+M "<mailcmd>"] [+L "<lpcmd>"] [+A <alert>]

           -c              syntax check alarm script file
           -G              run in debug mode
           -h              show program help
           -T              run in test mode
           +M              mail command
           +L              print command
           +A              alert

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include "common.h"

const char pikt_version[] = "PIKT pikt version " VERSION;

/* command-line options */
int ischeck = FALSE;
int isdebug = FALSE;
int isdoublecheck = FALSE;
int ishelp = FALSE;
int istest = FALSE;

int prescan;

int interactive = FALSE;

int uid = 0;		/* default to root */
char *master;
char *domain;
char *accessmode;

char hostname[MAXIDENT+1];
char domainname[MAXIDENT+1];

char **argvcurr;
char **argvlast;

unsigned stroffset = 0;

int lineno = 0;

char *altcurr;

char *altlockfile;
char *execlockfile;

int dohistory = FALSE;

int isprc = FALSE;

char *piktlog;
/* char syslog[] = "/var/log/syslog"; */

const char *leveltypes[] = { "EMERGENCY", "ALERT", "CRITICAL", "ERROR", "WARNING", "NOTICE", "INFO", "DEBUG" };
/* const char *leveltypes[] = { "EMERG", "ALERT", "CRIT", "ERR", "WARNING", "NOTICE", "INFO", "DEBUG" }; */

char *arytyp[] = { "ARYSTR", "ARYNUM" };	/* string exprinput prefixes
						   as a placeholder for an
						   array
						 */

char err[] = { ERR, '\0' };
char nil[] = { NIL, '\0' };
const char errstr[] = "<<ERR>>";
const char nilstr[] = "<<NIL>>";

const char dquote[] = { DQTPROXY, '\0' };
const char squote[] = { SQTPROXY, '\0' };
const char space[] = " ";

int exprtyp;

int exprres;
double exprval;
char *exprstr;

int vapndx = 0;

FILE *altin;
FILE *hstin;
FILE *hstout;

char *hsttmpfile;

char *mailcmd, *lpcmd;

alrm *alarms = (alrm *) NULL;

alrm *alarmcurr;
val *valcurr;
ky *scvarcurr;
ky *keycurr;
rul *rulcurr;
tok *tokcurr;
stm *stmcurr;

val *valnow;
val *valglb;
val *valhst;
val *valprc;
val *valtmp;

void loadalarms(char *);
void testalarms(char *);

int
main (int argc, char *argv[])
{
	init(argc, argv, (void (*)(void))pikt_usage);

	argvcurr = argv+1;	/* skip the program name */
	argvlast = argv+argc-1;
	cmdparse();		/* further parse the command line */

	altlockfile = (char *) malloc((strlen(etcdir)+strlen(altcurr)+11) * sizeof(char));
	sprintf(altlockfile, "%s/%s.alt.lock", etcdir, altcurr);
	execlockfile = (char *) malloc((strlen(etcdir)+strlen(altcurr)+12) * sizeof(char));
	sprintf(execlockfile, "%s/%s.exec.lock", etcdir, altcurr);
	piktlog = (char *) malloc((strlen(logdir)+strlen(altcurr)+6) * sizeof(char));
	sprintf(piktlog, "%s/%s.log", logdir, altcurr);

	/* apply the lock, to prevent other concurrent pikt sessions
	   for this alert
	 */
	if (islock(altlockfile)) {
		if (interactive) {
			fprintf(stderr, "%s found!  Another %s pikt run may be in progress.\nTry again later (else rm the file).\n", altlockfile, altcurr);
			exit(0);
		} else {
			/* logexit() automatically deletes the lock file,
			   so we do a logreturn() and exit() here instead
			 */
			logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "%s found, another %s pikt run may be in progress, run aborted.", altlockfile, altcurr);
			exit(0);
		}
	} else {
		if (mklock(altlockfile, (char *) NULL, interactive, TRUE) == ERR)
			logexit(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in main(), mklock() error for %s", altlockfile);
	}

	loadalarms(altcurr);

	if (!ischeck && !isdoublecheck)
		testalarms(altcurr);

	if (rmlocks(interactive, TRUE) == ERR)
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in main(), rmlocks() error");

	exit(0);

}

/* output program usage to screen
 */
void
pikt_usage (void)
{
	printf("\nUsage:  pikt [-cGhT] [+M \"<mailcmd>\"] [+L \"<lpcmd>\"] [+A <alert>]\n\n");
	printf("\t-c\t\tsyntax check alarm script file\n");
	printf("\t-G\t\trun in debug mode\n");
	printf("\t-h\t\tshow program help\n");
	printf("\t-T\t\trun in test mode\n");
	printf("\t+M\t\tmail command\n");
	printf("\t+L\t\tprint command\n");
	printf("\t+A\t\talert\n\n");
	return;
}

/* load and parse the alarm script file
 */
void
loadalarms (char *alert)
{
	char *altfile;

	if (isdoublecheck) {
		altfile = (char *) malloc((strlen(stgdir)+strlen(alert)+6) * sizeof(char));
		sprintf(altfile, "%s/%s.alt", stgdir, alert);
	} else {
		altfile = (char *) malloc((strlen(altdir)+strlen(alert)+6) * sizeof(char));
		sprintf(altfile, "%s/%s.alt", altdir, alert);
	}
	if ((altin = fopen(altfile, "r")) == NULL)
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in loadalarms(), fopen() error for %s", altfile);
	lineno = 1;
	alarmcurr = (alrm *) NULL;
	prescan = TRUE;
	altparse();
	rewind(altin);
	alarmcurr = (alrm *) NULL;
	prescan = FALSE;
	altparse();
	if (fclose(altin))	/* returns 0 on success */
		logreturn(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in loadalarms(), fclose() error for %s", altfile);
	return;
}

/* if the parse of the alarm scripts indicates that history processing is
   needed, do it; then, run each alarm script, one after the other; output
   to a new history file as required; send off any queued messages as e-mail
   or to the printer
 */
void
testalarms (char *alert)
{
	char *hstfile;
	struct stat fbuf;
	register int i;

	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "alert run begin");
	hstfile = (char *) malloc((strlen(hstdir)+strlen(alert)+6) * sizeof(char));
	sprintf(hstfile, "%s/%s.hst", hstdir, alert);
	if (dohistory) {
		if (stat(hstfile, &fbuf) != 0) {
			/* always true when alert run first time
			logreturn(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in testalarms(), stat() error for %s", hstfile);
			 */
			;
		} else {
			if (fbuf.st_size > 0) {
				if ((hstin = fopen(hstfile, "r")) == NULL)
					logreturn(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in testalarms(), fopen() error for %s", hstfile);
				else {
					lineno = 1;
					alarmcurr = (alrm *) NULL;
					keycurr = (ky *) NULL;
					hstparse();
					if (fclose(hstin))
						logreturn(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in testalarms(), fclose() error for %s", hstfile);
					makekeys();
				}
			}
		}
		hsttmpfile = (char *) malloc((strlen(hstdir)+strlen(alert)+6) * sizeof(char));
		sprintf(hsttmpfile, "%s/%s.tmp", hstdir, alert);
		if ((hstout = fopen(hsttmpfile, "w")) == NULL)
			logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in testalarms(), fopen() error for %s", hsttmpfile);
	} else {	/* no current history, so remove any old history file */
		if (stat(hstfile, &fbuf) == 0) {
			if (remove(hstfile))
				logreturn(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in testalarms(), remove() error for %s", hstfile);
		}
	}
	mailmsg = (msg *) NULL;
	printmsg = (msg *) NULL;
	newmailalert = TRUE;
	newprintalert = TRUE;
	foreach (alarmcurr, alarms) {
		if (isdebug)
			logreturn(PIKTLOG, DEBUG, __FILE__, __LINE__, FALSE, "ALARM %s", ALMNAM);
		newmailalarm = TRUE;
		newprintalarm = TRUE;
		if (STATUS == INACTIVE)
			continue;
		/* remove any potentially blocking exec lock file
		if (islock(execlockfile)) {
			if (rmlock(execlockfile, interactive, TRUE) == ERR)
				logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in testalarms(), rmlock() error for %s", execlockfile);
		}
		 */
		valnow = alarmcurr->values;
		valprc = alarmcurr->values->next;
		valglb = VALGLB;
		isprc = FALSE;	/* no preceding vals yet */
		if ((HSTTYP & (FIXEDVAL|FREEVAL)) && !HSTERR) {
			if (isdebug)
				dumpvals(VALHST);
		}
		for (i=0; i<=matches; i++) {
			if (valscn[i]) {
				free(valscn[i]);
				valscn[i] = (char *) NULL;
			}
		}
		matches = 0;
		if (scan(INPUTTYP, INPUTSRC, POSTYP) != ERR) {
			logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "ran %s", ALMNAM);
			/* we no longer need any of the vals--whether valnows,
			   valprcs, or history values, so let's free the
			   memory allocated to them; 
			   we only have valhsts to free if there was no error
			   when reading the history file
			 */
			if ((HSTTYP & (FIXEDVAL|FREEVAL)) && !HSTERR)
				freevalhsts(&VALHST);
			freevals(&valnow, FIXEDVAL|FREEVAL|BEGINVAL|ENDVAL, TRUE);
			freevals(&valprc, FIXEDVAL|FREEVAL|BEGINVAL|ENDVAL, TRUE);
			if (HSTTYP & (BEGINVAL|ENDVAL))
				freevals(&valglb, /* FIXEDVAL|FREEVAL| */ BEGINVAL|ENDVAL, TRUE);
			if (dohistory)
				fflush(hstout);
		}
	}
	sendmsg(mailmsg, mailcmd);
	freemsg(&mailmsg);
	sendmsg(printmsg, lpcmd);
	freemsg(&printmsg);
	if (dohistory) {
		if (fclose(hstout))
			logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in testalarms(), fclose() error for %s", hsttmpfile);
		if (remove(hstfile))
			/* always true when alert run first time
			logreturn(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in testalarms(), remove() error for %s", hstfile);
			 */
			;
		if (rename(hsttmpfile, hstfile))
			logreturn(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in testalarms(), rename() error for %s and %s", hsttmpfile, hstfile);
		chmod(hstfile, (mode_t)(S_IRUSR|S_IWUSR));	/* 600 */
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "alert run end");
	return;
}
